import java.io.*;
import java.awt.*;
import NSEditableObject;
import TclSnippet;

public class NSEntity extends NSEditableObject
{
    double x, y;

    public static final int NODE = 0;
    public static final int AGENT = 1;
    public static final int APPLICATION = 2;
    public static final int TIMER = 3;
    public static final int GENERIC = 4;
    public static final int NOMINAL_WIDTH = 612;       // = 8.5" x 72 dpi
    public static final int SIZE = 16;             // Size under scale = 1.0

    public NSEntity( TclSnippet inSnippet, String inName, double inX, double inY )
    {
	super( inSnippet, inName );
	
	setX( inX );
	setY( inY );
    }

    public void setX( double inX )
    {
	x = inX;
    }

    public double getX()
    {
	return x;
    }

    public void setY( double inY )
    {
	y = inY;
    }

    public double getY()
    {
	return y;
    }

    public void drawSelf( Graphics g, Dimension r)
    {
	Color color = Color.black;
	Color oc = Color.white;
	double scaleX = r.width;
	double scaleY = r.height;


	if (isSelected()) {
	    color = g.getColor();
	    g.setColor(Color.blue);
	}

	int dx = (int) Math.round( r.width * x);
	int dy = (int) Math.round( r.height * y);
	int size = (int) Math.round(NSEntity.SIZE * r.width / NSEntity.NOMINAL_WIDTH);
	int size2 = (int) Math.round( size / 2 );

	if (arrayIndex>=0) {
	    switch(getSnippet().getIcon()) {
	    case NSEntity.NODE:
		g . drawOval( dx - size2+2, dy - size2+2, size, size );
		g.setColor(Color.white);
		g . fillOval( dx - size2, dy - size2, size, size );
		break;
		
	    case NSEntity.AGENT:
		g . drawRect( dx - size2+2, dy - size2+2, size, size );
		g . drawLine( dx - size2+2, dy+2, dx + size2+2, dy+2 );
		g.setColor(Color.white);
		g . fillRect( dx - size2, dy - size2, size, size );
		break;
		
	    case NSEntity.APPLICATION:
		g . drawLine( dx, dy - size2+2, dx + size2, dy+2 );
		g . drawLine( dx + size2, dy+2, dx, dy + size2+2 );
		g . drawLine( dx, dy + size2+2, dx - size2, dy+2 );
		g . drawLine( dx - size2, dy+2, dx, dy - size2+2 );
		break;

	    case NSEntity.TIMER:
		g . drawOval( dx - size2+2, dy - size2+2, size, size );
		g . drawLine( dx + size2/2+2, dy - size2/2+2, dx+2, dy+2 );
		g . drawLine( dx + 2, dy + 2, dx + 2, dy+size2 + 2 );
		g.setColor(Color.white);
		g . fillOval( dx - size2, dy - size2, size, size );
		break;

	    case NSEntity.GENERIC:
		g . drawRect( dx - size2+2, dy - size2+2, size, size );
		g.setColor(Color.white);
		g . fillRect( dx - size2, dy - size2, size, size );
		break;
	    }
	}

	if (isSelected()) {
	    g.setColor(Color.blue);
	} else {
	    g.setColor(color);
	}

	switch(getSnippet().getIcon()) {
	case NSEntity.NODE:
	    g . drawOval( dx - size2, dy - size2, size, size );
	    break;

	case NSEntity.AGENT:
	    g . drawRect( dx - size2, dy - size2, size, size );
	    g . drawLine( dx - size2, dy, dx + size2, dy );
	    break;

	case NSEntity.APPLICATION:
	    g . drawLine( dx, dy - size2, dx + size2, dy );
	    g . drawLine( dx + size2, dy, dx, dy + size2 );
	    g . drawLine( dx, dy + size2, dx - size2, dy );
	    g . drawLine( dx - size2, dy, dx, dy - size2 );
	    break;

	case NSEntity.TIMER:
	    g . drawOval( dx - size2, dy - size2, size, size );
	    g . drawLine( dx + size2/2, dy - size2/2, dx, dy );
	    g . drawLine( dx, dy, dx, dy+size2 );
	    break;

	case NSEntity.GENERIC:
	    g . drawOval( dx - size2, dy - size2, size, size );
	    g . drawRect( dx - size2, dy - size2, size, size );
	    break;

	}

	g . drawString( getName(), dx + size, dy );
	if (isSelected())
	    g.setColor( color );
    }

    public boolean isHit( Dimension r, Point p )
    {
	double inX, inY, distance, size2 = (double) NSEntity.SIZE /NSEntity.NOMINAL_WIDTH / 2;
	
	inX = p.getX()/r.width;
	inY = p.getY()/r.height;
	distance = Math.sqrt( Math.pow(getX()-inX,2) + Math.pow(getY()-inY,2) );

	if (distance < size2) {
	    return true;
	} else 
	    return false;
    }

    public boolean isContained( Dimension r, Point p1, Point p2 )
    {
	double x1, y1, x2, y2;
	double minX, maxX, minY, maxY;

	x1 = p1.getX()/r.width;  
	y1 = p1.getY()/r.height;  
	x2 = p2.getX()/r.width;  
	y2 = p2.getY()/r.height;  
	
	if (x1<x2) {
	    minX = x1;
	    maxX = x2;
	} else {
	    minX = x2;
	    maxX = x1;
	}
	if (y1<y2) {
	    minY = y1;
	    maxY = y2;
	} else {
	    minY = y2;
	    maxY = y1;
	}

	if (getX() > minX && getX() < maxX && getY() > minY && getY() < maxY)
	    return true;
	else
	    return false;
    }

    public void moveBy( Dimension r, Dimension byWhat )
    {
	double dx, dy;

	dx = (double) byWhat.width/r.width;
	dy = (double) byWhat.height/r.height;

	setX( getX() + dx );
	setY( getY() + dy );
    }

    public String toString()
    {
	String s = "";

	s = super.toString();
	s = s + Double.toString(x) + "\n" + Double.toString(y) + "\n";

	return s;
    }

    public void fromString( BufferedReader br )
    {
	super.fromString( br );
	try {
	    setX( Double.parseDouble( br.readLine() ) );
	    setY( Double.parseDouble( br.readLine() ) );
	} catch (IOException ioe) {
	    System.out.println("Error reading the object: " + ioe.toString());
	}
    }
}
