// TclExpression.java
//
// Any object that can aid in building a tcl string.

import NSObject;
import NSRelation;
import java.util.*;

public class TclSnippet extends Object
{
    String name, base;
    public boolean isRelation, isUnique;
    int icon;
    String fromBase, toBase;
    public boolean isFromBaseUnique, isToBaseUnique;
    int baseStyle, lineStyle, lineWidth, endStyle;
    ArrayList attributes, patterns;

    static final int SOLID = 0;
    static final int DASHED = 1;
    static final int DOTTED = 2;

    public TclSnippet( String s )
    {
	attributes = new ArrayList();
	patterns = new ArrayList();
	parseSelf(s);
    }

    void parseSelf( String s )
    {
	int headEndIdx, beginIdx, endIdx;
	String head, attr, patt, a;
	String instanceStr;
	
	headEndIdx = s.indexOf(':');
	beginIdx = s.indexOf("begin");
	endIdx = s.indexOf("end");

	head = s.substring(0,headEndIdx).trim();
	// System.out.println("HEAD:\n"+head);
	attr = s.substring(headEndIdx+1,beginIdx).trim();
	//	System.out.println("ATTRIBUTES:\n"+attr);
	patt = s.substring(beginIdx+5,endIdx).trim();
	//	System.out.println("PATTERNS:\n"+patt);

	StringTokenizer st = new StringTokenizer( head );
	a = st.nextToken();
	if (a.indexOf('!')>=0) {
	    isUnique = true;
	    a = a.substring(1);
	} else {
	    isUnique = false;
	}
				   
	isRelation = a.equals("relation");
	st.nextToken();
	setName(st.nextToken());
	if (isRelation) {
	    a = st.nextToken();
	    if (a.indexOf('!')>=0) {
		a = a.substring(1);
		isFromBaseUnique = true;
	    } else {
		isFromBaseUnique = false;
	    }
	    setFromBase(a);
	    a = st.nextToken();
	    if (a.indexOf('!')>=0) {
		a = a.substring(1);
		isToBaseUnique = true;
	    } else {
		isToBaseUnique = false;
	    }
	    setToBase(a);
	    setBaseStyle(Integer.parseInt(st.nextToken()));
	    setLineStyle(Integer.parseInt(st.nextToken()));
	    setLineWidth(Integer.parseInt(st.nextToken()));
	    setEndStyle(Integer.parseInt(st.nextToken()));
	    //   System.out.println("Read relation snippet: " + baseStyle + "," + lineStyle + "," + lineWidth + "," + endStyle );
	} else {
	    setBase(st.nextToken());
	    setIcon(Integer.parseInt(st.nextToken()));
	}

	// Now, read the attribute definitions.
	
	int iStart = 0, iEnd;
	while ((iEnd = attr . indexOf(';',iStart))>=0) {
	    attributes.add(new TclAttribute( attr.substring(iStart,iEnd) ) );
	    iStart = iEnd+1;
	}

	// Finally, read the patterns.

	iStart = 0;
	while ((iEnd = patt . indexOf(';',iStart))>=0) {
	    patterns.add(new TclPattern( patt.substring(iStart,iEnd) ) );
	    iStart = iEnd+1;
	}
    }

    // Gets & sets for all members
    public void setName( String inName )
    {
	name = inName;
    }

    public String getName()
    {
	return name;
    }

    public void setBase( String inBase )
    {
	base = inBase;
    }

    public String getBase()
    {
	return base;
    }

    public void setIcon( int inIcon )
    {
	icon = inIcon;
    }

    public int getIcon()
    {
	return icon;
    }

    public void setFromBase( String inFromBase )
    {
	fromBase = inFromBase;
    }

    public String getFromBase()
    {
	return fromBase;
    }

    public void setToBase( String inToBase )
    {
	toBase = inToBase;
    }

    public String getToBase()
    {
	return toBase;
    }

    public int getBaseStyle() {
	return baseStyle;
    }
    
    public void setBaseStyle(int  inBaseStyle) {
	baseStyle = inBaseStyle;
    }

    public int getLineStyle() {
	return lineStyle;
    }
    
    public void setLineStyle(int  inLineStyle) {
	lineStyle = inLineStyle;
    }

    public int getLineWidth() {
	return lineWidth;
    }
    
    public void setLineWidth(int  inLineWidth) {
	lineWidth = inLineWidth;
    }
    
    public int getEndStyle() {
	return endStyle;
    }
    
    public void setEndStyle(int  inEndStyle) {
	endStyle = inEndStyle;
    }

    public int getAttributeCount()
    {
	return attributes.size();
    }

    public TclAttribute getAttribute( int inIndex )
    {
	if (inIndex>=0 && inIndex<attributes.size())
	    return (TclAttribute) (attributes.get(inIndex));
	else
	    return null;
    }

    public String toString()
    {
	String s;
	int i;
	
	s = getName();
	for(i=0;i<attributes.size();i++)
	    s = s + "  " + attributes.get(i).toString() + "\n";
	s = s + "begin\n";
	for(i=0;i<patterns.size();i++)
	    s = s + "  " + patterns.get(i).toString() + "\n";
	s = s + "end\n";
	return s;
    }

    public void instantiateNSObject( NSObject o )
    {
	int i;

	for(i=0;i<attributes.size(); i++) {
	    o . setAttribute(i,((TclAttribute)attributes.get(i)).defaultValue);
	}
    }

    public String toTcl( NSWorld w, NSObject o, char sep )
    {
	String preamble = "", s, epilogue = "", aname, aname2;
	TclPattern p;
	NSRelation ro;
	NSArray a;
	int i, i2,asize,asize2;
	
	// Get the preamble and epilogue for the snippet.
	if (o.getArrayIndex()>=0) {
	    a = w.getArray(o.getArrayIndex());
	    aname = a.name;
	    asize = a.elements;
	    preamble = "for {set "+aname+" 0} {$"+aname+"<"+Integer.toString(asize)+"} {incr " + aname + "} {\n";
	    epilogue = "}\n";
	} else {
	    if (isRelation) {
		ro = (NSRelation) o;
		i = ro.getFrom().getArrayIndex(); i2 = ro.getTo().getArrayIndex();
		if (i>=0 || i2>=0) {
		    if (i==i2) {
			aname = w.getArray(i).name;
			asize = w.getArray(i).elements;
			preamble = "for {set "+aname+" 0} {$"+aname+"<"+Integer.toString(asize)+"} {incr "+aname+"} {\n";
			epilogue = "}\n";
		    } else {
			if (i>=0) {
			    aname = w.getArray(i).name;
			    asize = w.getArray(i).elements;
			    preamble = "for {set "+aname+" 0} {$"+aname+"<"+Integer.toString(asize)+"} {incr "+aname+"} {\n";
			    epilogue = "}\n";
			}
			if (i2>=0) {
			    aname = w.getArray(i2).name;
			    asize = w.getArray(i2).elements;
			    preamble = preamble+"  for {set "+aname+" 0} {$"+aname+"<"+Integer.toString(asize)+"} {incr "+aname+"} {\n";
			    epilogue = epilogue + "  }\n";
			}
		    }
		} 
	    }
	}

	String sApp;
	// Now pattern substitution
	s = "";
	for (i=0; i<patterns.size(); i++) {
	    p = (TclPattern) patterns.get(i);
	    if (p.isConditional) {
		if (p.attributeValue.equals(valueOf(w,o,p.attribute)))
		    sApp = patternToTcl(p.pattern,w,o,sep);
		else
		    sApp = patternToTcl(p.alternativePattern,w,o,sep);
	    } else {
		sApp = patternToTcl(p.pattern,w,o,sep);
	    }
	    if (!sApp.trim().equals(""))
		s = s + sApp + "\n";
	    
	}
	if (s.trim().equals(""))
	    return "";
	else
	    return (preamble + s + epilogue).trim();
    }

    String patternToTcl( String pattern, NSWorld w, NSObject o, char sep )
    {
	int i,iLast = 0;
	String sNew = "";
	
	while (( i = pattern . indexOf(sep,iLast))>=0) {
	    sNew = sNew + pattern . substring( iLast, i );
	    iLast = pattern . indexOf(sep,i+1);
	    if ( iLast < 0 ) 
		return "ERROR: Bad formed pattern.";
	    sNew = sNew + valueOf( w, o, pattern . substring(i+1,iLast));
	    iLast++;
	}
	if (iLast<pattern.length())
	    sNew = sNew + pattern . substring(iLast);
	return sNew;
    }

    String valueOf( NSWorld w, NSObject o, String attrName )
    {
	int i;

	// Special field values
	i = attrName.indexOf("env");
	if (i>=0) {
	    return w.getEnvironment().getSnippet().
		valueOf(w,w.getEnvironment(),attrName.substring(i+4));
	}
	if (attrName.equals("from"))
	    return arrayedName( ((NSRelation) o).getFrom(), w );
	if (attrName.equals("to"))
	    return arrayedName( ((NSRelation) o).getTo(), w );

	//  It's an array variable
	if (attrName.equals("name")) {
	    return arrayedName( o, w );
	}
	
	// Variable attributes
	for (i=0;i<attributes.size();i++)
	    if (((TclAttribute)attributes.get(i)).name.equals(attrName))
		return o.getAttribute(i);
	return attrName+".NotFound";
    }
    
    public String arrayedName( NSObject o, NSWorld w )
    {
	if (o.getArrayIndex()>=0)
	    return o.getName() + "($" + w.getArray(o.getArrayIndex()).name + ")";
	else
	    return o.getName();
    }
}







