import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import NSWorld;
import NSObject;
import DMModel;
import SObjectBrowser;
import SToolBar;

public class NSModel extends NSWorld implements DMModel
{
    SToolBar toolBox;
    DMView editView;
    JTextArea tclView;
    SObjectBrowser objectPanel;
    NSWorldView worldView;
    boolean isDirty;
    

    public NSModel( NSObject inEnv, SToolBar inTBar )
    {
		super( inEnv );
		toolBox = inTBar;
		isDirty = false;
    }

    public void setViews( DMView inEView, JTextArea inTclView,  SObjectBrowser inObjectPanel, NSWorldView inWorldView )
    {
		editView = inEView;
		tclView = (JTextArea) inTclView;
		objectPanel = inObjectPanel;
		worldView = inWorldView;
    }

    public int getEditionMode()
    {
		String inst;
		TclSnippet s = toolBox . getSelectedSnippet();
		
		if (s==null)
		    return DMModel.SELECT_MODE;
	
		if (s . isRelation)
		    return DMModel.RELATION_MODE;
		else
		    return DMModel.ICON_MODE;
    }

    public void addSimpleObject(Dimension r, Point p, boolean unselectOther)
    {
		String inst, newName;
		TclSnippet s = toolBox . getSelectedSnippet();
		NSObject o;
		double x, y;

		x = p.getX()/r.width;
		y = p.getY()/r.height;
		
		if (s==null)
		    return;

		// Check for uniqueness
		if (s.isUnique && objectOfClassExists( s.getName() ))
		    return;

		isDirty = true;
		// Get a new unique generic name
		int i=-1;
		do {
		    i++;
		    newName = s.getName()+i;
		} while (!isValidName(newName));
	
		o = new NSEntity( s, newName, x, y);
		s . instantiateNSObject( o );
		addObject( o );
		tclView . setText( toTcl() );
		worldView . updateList();
    }

    public void addRelationObject( DMObject oFrom, DMObject oTo, boolean unSelectOthers )
    {
		NSObject oF, oT;
		String inst, newName;
		int i;
		TclSnippet s = toolBox . getSelectedSnippet();
		NSRelation o;
		
		if (s==null || !s.isRelation)
		    return;
		
		if (s.isFromBaseUnique && relationOfClassExists(oFrom,s.getName())) {
		    return;
		}
		if (s.isToBaseUnique && relationOfClassExists(oTo,s.getName())) {
		    return;
		}
	
		isDirty = true;
		i=-1;
		do {
		    i++;
		    newName = s.getName()+i;
		} while (!isValidName(newName));
	
		if (((NSEntity) oFrom).getSnippet().getBase().equals( s.getFromBase() ) &&
		    ((NSEntity)  oTo).getSnippet().getBase().equals( s.getToBase() )) {
		    o = new NSRelation( s, newName, (NSEntity) oFrom, (NSEntity) oTo );
		    s . instantiateNSObject( o );
		    addObject( o );
		    tclView . setText( toTcl() );
		    worldView . updateList();
		}
    }
    	
    public void updateAllViews()
    {
		editView . repaint();
		objectPanel . selectionChanged();
		tclView . setText( toTcl() );
    }
    
    boolean objectOfClassExists( String theClass )
    {
		NSObject o;
		int i;

		for (i=0; i<getSize(); i++) {
		    o = (NSObject)getObjectAt(i);
		    if (o.getSnippet().getName().equals(theClass)) 
			return true;
		}
		return false;
    }

    boolean relationOfClassExists( DMObject dmo, String theClass )
    {
		NSObject o, O;
		NSRelation or;
		int i;

		O = (NSObject) dmo;
		for (i=0; i<getSize(); i++) {
		    o = (NSObject)getObjectAt(i);
		    if (o.getSnippet().isRelation) {
				or = (NSRelation) o;
				if (or.getFrom()==O && or.getSnippet().getName().equals(theClass) ||
				    or.getTo()==O && or.getSnippet().getName().equals(theClass)) {
				    return true;
				}
			}
		}
		return false;
    }

    public int getSize()
    {
		return objects.size();
    }

    public DMObject getObjectAt( int inIndex )
    {
		if (inIndex>=0 && inIndex<objects.size())
		    return (DMObject) (objects.get(inIndex));
		else
		    return null;
    }

    public void removeSelected()
    {
		int i;
		NSEditableObject o;
		NSRelation or;
	
		// First, select also arcs attached to a entity to be deleted
		for (i=0; i<getSize(); i++) {
		    o = (NSEditableObject)getObjectAt(i);
		    if (o.getSnippet().isRelation) {
				or = (NSRelation) o;
				if (or.getFrom().isSelected() || or.getTo().isSelected()) {
				    or.select();
				}
		    }
		}
		
		editView.repaint();
	
		Iterator iter = objects.iterator();

		iter.next();
		while (iter.hasNext()) {
		    o = (NSEditableObject)iter.next();
		    if (o.isSelected()) {
		    	isDirty = true;
				iter.remove();
		    }
		}

		updateAllViews();
    }

    public void newModel()
    {
		// Clean arrays
		while (arrays.size()>0) 
			arrays.remove(0);
	
		Iterator iter = objects.iterator();
	
		iter.next();
		while (iter.hasNext()) {
			iter.next();
			iter.remove();
		}
	
		((NSObject) getObjectAt(0)).getSnippet().instantiateNSObject((NSObject)getObjectAt(0));
		((NSObject) getObjectAt(0)).setName("ns");
		updateAllViews();
		setDirty(false);
    }

    public String toTcl()
    {
		int i;
		String s = "";
	
		for (i=0; i<getSize(); i++) 			
			s = s + ((NSObject)getObjectAt(i)).getSnippet().toTcl(this,(NSObject)getObjectAt(i),'#') + '\n';
			
		s = s + "$" + ((NSObject)getObjectAt(0)).getName() + " run";
		return s;
    }

    public String toString()
    {
		String s = "";
		int i;
		  
		s = Integer.toString(getArrayCount()) + "\n";
		for (i=0; i<getArrayCount(); i++) {
			s = s + getArray(i).toString();
		}
		s = s + Integer.toString(objects.size()) + "\n";
		for (i=0; i<objects.size(); i++) {
			s = s + ((NSObject)getObjectAt(i)).toString();
		}
		return s;
    }
    
    public boolean dirty() 
    {
    	return isDirty;
    }
    
    public void setDirty( boolean dirtyState ) 
    {
    	isDirty = dirtyState;
    }
}









