import TclSnippet;
import NSEditableObject;
import NSEntity;
import java.awt.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.*;

class NSRelation extends NSEditableObject implements DMObject
{
    NSEntity from, to;
    public static final double SELECTION_RADIUS = 2.0;

    public NSRelation( TclSnippet inSnippet, String inName, NSEntity inFrom, NSEntity inTo )
    {
	super(inSnippet, inName);
	from = inFrom;
	to = inTo;
    }

    public NSEntity getFrom()
    {
	return from;
    }

    public NSEntity getTo()
    {
	return to;
    }
    
    public void drawSelf( Graphics g, Dimension r )
    {
	Graphics2D g2 = (Graphics2D) g;
	Color color = Color.black;
	if (isSelected()) {
	    color = g.getColor();
	    g.setColor(Color.blue);
	}
	int baseStyle, lineStyle, lineWidth, endStyle;

	baseStyle = getSnippet().getBaseStyle();
	endStyle = getSnippet().getEndStyle();
	lineStyle = getSnippet().getLineStyle();
	lineWidth = getSnippet().getLineWidth();
	int aSize = lineWidth * 2;

	// Get dimensions and stuff
	double scale = r.width / NSEntity.NOMINAL_WIDTH;
	double size = (double) NSEntity.SIZE / NSEntity.NOMINAL_WIDTH;
	double size2 = size/2.0;

	// Get start and end points
	double x1, x2, y1, y2, xa1, ya1, xa2, ya2;

	double ux, uy, vsize;
	double dx, dy; 
	float dash[] = new float[2];
	
	dx = getTo().getX() - getFrom().getX();
	dy = getTo().getY() - getFrom().getY();
	
	vsize = Math.sqrt( dx*dx + dy*dy );
	ux = dx/vsize;
	uy = dy/vsize;

	x1 = getFrom().getX() + size2 * ux;
	y1 = getFrom().getY() + size2 * uy;
	x2 = getTo().getX() - size2 * ux;
	y2 = getTo().getY() - size2 * uy;
	
	// Now, create the stroke

	BasicStroke pen;

	switch (lineStyle) {
	case TclSnippet.SOLID:
	    pen = new BasicStroke( lineWidth );
	    break;
	case TclSnippet.DASHED:
	    dash[0]=3;
	    dash[1]=2;
	    pen = new BasicStroke( (float) lineWidth, 0, 0, 1, dash, 0 );
	    break;
	case TclSnippet.DOTTED:
	default:
	    dash[0]=1;
	    dash[1]=5;
	    pen = new BasicStroke( (float) lineWidth, 0, 0, 1, dash, 0 );
	    break;
	}
	g2 . setStroke( pen );

	// Draw the freekin' line

	g2 . drawLine( (int)Math.round(x1*r.width), (int)Math.round(y1 * r.height),
		       (int)Math.round(x2*r.width), (int)Math.round(y2 * r.height));

	g2.setStroke( new BasicStroke( lineWidth ) );
	switch (baseStyle) {
	case 0:
	    break;
	case 1:
	    g2 . drawLine( (int)Math.round(x1*r.width), (int)Math.round(y1 * r.height),
			   (int)Math.round(x1*r.width+2*ux-2*uy), (int)Math.round(y1 * r.height+2*uy+aSize*ux));
	    g2 . drawLine( (int)Math.round(x1*r.width), (int)Math.round(y1 * r.height),
			   (int)Math.round(x1*r.width+2*ux+2*uy), (int)Math.round(y1 * r.height+2*uy-aSize*ux));
	    break;
	case 2:
	    g2.fillRect( (int)Math.round(x1*r.width)-aSize-1, (int)Math.round(y1 * r.height)-aSize-1,(aSize+1)*2,(aSize+1)*2);
	    break;
	case 3:
	default:
	    g2.fillOval( (int)Math.round(x1*r.width)-aSize-1, (int)Math.round(y1 * r.height)-aSize-1,(aSize+1)*2,(aSize+1)*2);
	    break;
	}

	switch (endStyle) {
	case 0:
	    break;
	case 1:
	    g2 . drawLine( (int)Math.round(x2*r.width), (int)Math.round(y2 * r.height),
			   (int)Math.round(x2*r.width-aSize*ux-aSize*uy), (int)Math.round(y2 * r.height-aSize*uy+aSize*ux));
	    g2 . drawLine( (int)Math.round(x2*r.width), (int)Math.round(y2 * r.height),
			   (int)Math.round(x2*r.width-aSize*ux+aSize*uy), (int)Math.round(y2 * r.height-aSize*uy-aSize*ux));
	    break;
	case 2:
	    g2.drawRect( (int)Math.round(x1*r.width)-aSize, (int)Math.round(y1 * r.height)-aSize,2*aSize+1,2*aSize+1);
	    break;
	case 3:
	default:
	    g2.drawOval( (int)Math.round(x1*r.width)-aSize, (int)Math.round(y1 * r.height)-aSize,2*aSize+1,2*aSize+1);
	    break;
	}

	g2 . setStroke( new BasicStroke() );
	
	if (isSelected()) 
	    g.setColor(color);
    }

    public void moveBy( Dimension r, Dimension byWhat )
    {
	if (!getFrom().isSelected())
	    getFrom().select();
	if (!getTo().isSelected())
	    getTo().select();
    }
    public boolean isHit( Dimension r, Point p )
    {
	double t0;
	double x1, y1, x2, y2, xp, yp;
	double inX = p.getX()/r.width;
	double inY = p.getY()/r.height;
	double radius = NSRelation.SELECTION_RADIUS / NSEntity.NOMINAL_WIDTH;

	x1 = getFrom().getX();
	y1 = getFrom().getY();
	x2 = getTo().getX();
	y2 = getTo().getY();
	
	t0 = ((inX-x1)*(x2-x1)+(inY-y1)*(y2-y1))/
	    (Math.pow(x2-x1,2)+Math.pow(y2-y1,2));
	if (t0>0 && t0<1) {
	    xp = x1 + t0*(x2-x1);
	    yp = y1 + t0*(y2-y1);
	    if (Math.sqrt(Math.pow(inX-xp,2)+Math.pow(inY-yp,2))<radius) 
		return true;
	}
	return false;
    }
	
    public boolean isContained( Dimension r, Point p1, Point p2 )
    {
	if (getFrom().isContained( r, p1, p2 ) && getTo().isContained( r, p1, p2 ))
	    return true;
	else
	    return false;
    }

    public String toString()
    {
	String str;
	int i;
	
	str = super.toString();
	str = str + from.getName() + "\n" + to.getName() + "\n";
	return str;
    }

    public void fromString( BufferedReader br, NSModel M )
    {
	try {
	    super . fromString( br );
	    from = (NSEntity) M . getObject( br.readLine() );
	    to = (NSEntity) M . getObject( br.readLine() );
	} catch( IOException ioe ) {
	    System.out.println("Error reading relation object: " + ioe.toString() );
	}
    }

} // NSRelation definition
